/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A common class for general resource information
 *
 * @extends models['Resource']
 */
class LocalNetworkGateway extends models['Resource'] {
  /**
   * Create a LocalNetworkGateway.
   * @property {object} [localNetworkAddressSpace] Local network site address
   * space.
   * @property {array} [localNetworkAddressSpace.addressPrefixes] A list of
   * address blocks reserved for this virtual network in CIDR notation.
   * @property {string} [gatewayIpAddress] IP address of local network gateway.
   * @property {object} [bgpSettings] Local network gateway's BGP speaker
   * settings.
   * @property {number} [bgpSettings.asn] The BGP speaker's ASN.
   * @property {string} [bgpSettings.bgpPeeringAddress] The BGP peering address
   * and BGP identifier of this BGP speaker.
   * @property {number} [bgpSettings.peerWeight] The weight added to routes
   * learned from this BGP speaker.
   * @property {string} [resourceGuid] The resource GUID property of the
   * LocalNetworkGateway resource.
   * @property {string} [provisioningState] The provisioning state of the
   * LocalNetworkGateway resource. Possible values are: 'Updating', 'Deleting',
   * and 'Failed'.
   * @property {string} [etag] A unique read-only string that changes whenever
   * the resource is updated.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of LocalNetworkGateway
   *
   * @returns {object} metadata of LocalNetworkGateway
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'LocalNetworkGateway',
      type: {
        name: 'Composite',
        className: 'LocalNetworkGateway',
        modelProperties: {
          id: {
            required: false,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          localNetworkAddressSpace: {
            required: false,
            serializedName: 'properties.localNetworkAddressSpace',
            type: {
              name: 'Composite',
              className: 'AddressSpace'
            }
          },
          gatewayIpAddress: {
            required: false,
            serializedName: 'properties.gatewayIpAddress',
            type: {
              name: 'String'
            }
          },
          bgpSettings: {
            required: false,
            serializedName: 'properties.bgpSettings',
            type: {
              name: 'Composite',
              className: 'BgpSettings'
            }
          },
          resourceGuid: {
            required: false,
            serializedName: 'properties.resourceGuid',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          etag: {
            required: false,
            serializedName: 'etag',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = LocalNetworkGateway;
