/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Information about packet capture session.
 *
 */
class PacketCaptureResult {
  /**
   * Create a PacketCaptureResult.
   * @property {string} [name] Name of the packet capture session.
   * @property {string} [id] ID of the packet capture operation.
   * @property {string} [etag] A unique read-only string that changes whenever
   * the resource is updated. Default value: 'A unique read-only string that
   * changes whenever the resource is updated.' .
   * @property {string} target The ID of the targeted resource, only VM is
   * currently supported.
   * @property {number} [bytesToCapturePerPacket] Number of bytes captured per
   * packet, the remaining bytes are truncated. Default value: 0 .
   * @property {number} [totalBytesPerSession] Maximum size of the capture
   * output. Default value: 1073741824 .
   * @property {number} [timeLimitInSeconds] Maximum duration of the capture
   * session in seconds. Default value: 18000 .
   * @property {object} storageLocation Describes the storage location for a
   * packet capture session.
   * @property {string} [storageLocation.storageId] The ID of the storage
   * account to save the packet capture session. Required if no local file path
   * is provided.
   * @property {string} [storageLocation.storagePath] The URI of the storage
   * path to save the packet capture. Must be a well-formed URI describing the
   * location to save the packet capture.
   * @property {string} [storageLocation.filePath] A valid local path on the
   * targeting VM. Must include the name of the capture file (*.cap). For linux
   * virtual machine it must start with /var/captures. Required if no storage
   * ID is provided, otherwise optional.
   * @property {array} [filters] A list of packet capture filters.
   * @property {string} [provisioningState] The provisioning state of the
   * packet capture session. Possible values include: 'Succeeded', 'Updating',
   * 'Deleting', 'Failed'
   */
  constructor() {
  }

  /**
   * Defines the metadata of PacketCaptureResult
   *
   * @returns {object} metadata of PacketCaptureResult
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'PacketCaptureResult',
      type: {
        name: 'Composite',
        className: 'PacketCaptureResult',
        modelProperties: {
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          etag: {
            required: false,
            serializedName: 'etag',
            defaultValue: 'A unique read-only string that changes whenever the resource is updated.',
            type: {
              name: 'String'
            }
          },
          target: {
            required: true,
            serializedName: 'properties.target',
            type: {
              name: 'String'
            }
          },
          bytesToCapturePerPacket: {
            required: false,
            serializedName: 'properties.bytesToCapturePerPacket',
            defaultValue: 0,
            type: {
              name: 'Number'
            }
          },
          totalBytesPerSession: {
            required: false,
            serializedName: 'properties.totalBytesPerSession',
            defaultValue: 1073741824,
            type: {
              name: 'Number'
            }
          },
          timeLimitInSeconds: {
            required: false,
            serializedName: 'properties.timeLimitInSeconds',
            defaultValue: 18000,
            type: {
              name: 'Number'
            }
          },
          storageLocation: {
            required: true,
            serializedName: 'properties.storageLocation',
            type: {
              name: 'Composite',
              className: 'PacketCaptureStorageLocation'
            }
          },
          filters: {
            required: false,
            serializedName: 'properties.filters',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'PacketCaptureFilterElementType',
                  type: {
                    name: 'Composite',
                    className: 'PacketCaptureFilter'
                  }
              }
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = PacketCaptureResult;
