/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Public IP prefix resource.
 *
 * @extends models['Resource']
 */
class PublicIPPrefix extends models['Resource'] {
  /**
   * Create a PublicIPPrefix.
   * @property {object} [sku] The public IP prefix SKU.
   * @property {string} [sku.name] Name of a public IP prefix SKU. Possible
   * values include: 'Standard'
   * @property {string} [publicIPAddressVersion] The public IP address version.
   * Possible values include: 'IPv4', 'IPv6'
   * @property {array} [ipTags] The list of tags associated with the public IP
   * prefix.
   * @property {number} [prefixLength] The Length of the Public IP Prefix.
   * @property {string} [ipPrefix] The allocated Prefix
   * @property {array} [publicIPAddresses] The list of all referenced
   * PublicIPAddresses
   * @property {string} [resourceGuid] The resource GUID property of the public
   * IP prefix resource.
   * @property {string} [provisioningState] The provisioning state of the
   * Public IP prefix resource. Possible values are: 'Updating', 'Deleting',
   * and 'Failed'.
   * @property {string} [etag] A unique read-only string that changes whenever
   * the resource is updated.
   * @property {array} [zones] A list of availability zones denoting the IP
   * allocated for the resource needs to come from.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of PublicIPPrefix
   *
   * @returns {object} metadata of PublicIPPrefix
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'PublicIPPrefix',
      type: {
        name: 'Composite',
        className: 'PublicIPPrefix',
        modelProperties: {
          id: {
            required: false,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          sku: {
            required: false,
            serializedName: 'sku',
            type: {
              name: 'Composite',
              className: 'PublicIPPrefixSku'
            }
          },
          publicIPAddressVersion: {
            required: false,
            serializedName: 'properties.publicIPAddressVersion',
            type: {
              name: 'String'
            }
          },
          ipTags: {
            required: false,
            serializedName: 'properties.ipTags',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'IpTagElementType',
                  type: {
                    name: 'Composite',
                    className: 'IpTag'
                  }
              }
            }
          },
          prefixLength: {
            required: false,
            serializedName: 'properties.prefixLength',
            type: {
              name: 'Number'
            }
          },
          ipPrefix: {
            required: false,
            serializedName: 'properties.ipPrefix',
            type: {
              name: 'String'
            }
          },
          publicIPAddresses: {
            required: false,
            serializedName: 'properties.publicIPAddresses',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ReferencedPublicIpAddressElementType',
                  type: {
                    name: 'Composite',
                    className: 'ReferencedPublicIpAddress'
                  }
              }
            }
          },
          resourceGuid: {
            required: false,
            serializedName: 'properties.resourceGuid',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          etag: {
            required: false,
            serializedName: 'etag',
            type: {
              name: 'String'
            }
          },
          zones: {
            required: false,
            serializedName: 'zones',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = PublicIPPrefix;
