/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A common class for general resource information
 *
 * @extends models['Resource']
 */
class VirtualNetworkGatewayConnection extends models['Resource'] {
  /**
   * Create a VirtualNetworkGatewayConnection.
   * @property {string} [authorizationKey] The authorizationKey.
   * @property {object} virtualNetworkGateway1 The reference to virtual network
   * gateway resource.
   * @property {array} [virtualNetworkGateway1.ipConfigurations] IP
   * configurations for virtual network gateway.
   * @property {string} [virtualNetworkGateway1.gatewayType] The type of this
   * virtual network gateway. Possible values are: 'Vpn' and 'ExpressRoute'.
   * Possible values include: 'Vpn', 'ExpressRoute'
   * @property {string} [virtualNetworkGateway1.vpnType] The type of this
   * virtual network gateway. Possible values are: 'PolicyBased' and
   * 'RouteBased'. Possible values include: 'PolicyBased', 'RouteBased'
   * @property {boolean} [virtualNetworkGateway1.enableBgp] Whether BGP is
   * enabled for this virtual network gateway or not.
   * @property {boolean} [virtualNetworkGateway1.activeActive] ActiveActive
   * flag
   * @property {object} [virtualNetworkGateway1.gatewayDefaultSite] The
   * reference of the LocalNetworkGateway resource which represents local
   * network site having default routes. Assign Null value in case of removing
   * existing default site setting.
   * @property {string} [virtualNetworkGateway1.gatewayDefaultSite.id] Resource
   * ID.
   * @property {object} [virtualNetworkGateway1.sku] The reference of the
   * VirtualNetworkGatewaySku resource which represents the SKU selected for
   * Virtual network gateway.
   * @property {string} [virtualNetworkGateway1.sku.name] Gateway SKU name.
   * Possible values include: 'Basic', 'HighPerformance', 'Standard',
   * 'UltraPerformance', 'VpnGw1', 'VpnGw2', 'VpnGw3', 'VpnGw1AZ', 'VpnGw2AZ',
   * 'VpnGw3AZ', 'ErGw1AZ', 'ErGw2AZ', 'ErGw3AZ'
   * @property {string} [virtualNetworkGateway1.sku.tier] Gateway SKU tier.
   * Possible values include: 'Basic', 'HighPerformance', 'Standard',
   * 'UltraPerformance', 'VpnGw1', 'VpnGw2', 'VpnGw3', 'VpnGw1AZ', 'VpnGw2AZ',
   * 'VpnGw3AZ', 'ErGw1AZ', 'ErGw2AZ', 'ErGw3AZ'
   * @property {number} [virtualNetworkGateway1.sku.capacity] The capacity.
   * @property {object} [virtualNetworkGateway1.vpnClientConfiguration] The
   * reference of the VpnClientConfiguration resource which represents the P2S
   * VpnClient configurations.
   * @property {object}
   * [virtualNetworkGateway1.vpnClientConfiguration.vpnClientAddressPool] The
   * reference of the address space resource which represents Address space for
   * P2S VpnClient.
   * @property {array}
   * [virtualNetworkGateway1.vpnClientConfiguration.vpnClientAddressPool.addressPrefixes]
   * A list of address blocks reserved for this virtual network in CIDR
   * notation.
   * @property {array}
   * [virtualNetworkGateway1.vpnClientConfiguration.vpnClientRootCertificates]
   * VpnClientRootCertificate for virtual network gateway.
   * @property {array}
   * [virtualNetworkGateway1.vpnClientConfiguration.vpnClientRevokedCertificates]
   * VpnClientRevokedCertificate for Virtual network gateway.
   * @property {array}
   * [virtualNetworkGateway1.vpnClientConfiguration.vpnClientProtocols]
   * VpnClientProtocols for Virtual network gateway.
   * @property {array}
   * [virtualNetworkGateway1.vpnClientConfiguration.vpnClientIpsecPolicies]
   * VpnClientIpsecPolicies for virtual network gateway P2S client.
   * @property {string}
   * [virtualNetworkGateway1.vpnClientConfiguration.radiusServerAddress] The
   * radius server address property of the VirtualNetworkGateway resource for
   * vpn client connection.
   * @property {string}
   * [virtualNetworkGateway1.vpnClientConfiguration.radiusServerSecret] The
   * radius secret property of the VirtualNetworkGateway resource for vpn
   * client connection.
   * @property {object} [virtualNetworkGateway1.bgpSettings] Virtual network
   * gateway's BGP speaker settings.
   * @property {number} [virtualNetworkGateway1.bgpSettings.asn] The BGP
   * speaker's ASN.
   * @property {string} [virtualNetworkGateway1.bgpSettings.bgpPeeringAddress]
   * The BGP peering address and BGP identifier of this BGP speaker.
   * @property {number} [virtualNetworkGateway1.bgpSettings.peerWeight] The
   * weight added to routes learned from this BGP speaker.
   * @property {object} [virtualNetworkGateway1.customRoutes] The reference of
   * the address space resource which represents the custom routes address
   * space specified by the the customer for virtual network gateway and
   * VpnClient.
   * @property {array} [virtualNetworkGateway1.customRoutes.addressPrefixes] A
   * list of address blocks reserved for this virtual network in CIDR notation.
   * @property {string} [virtualNetworkGateway1.resourceGuid] The resource GUID
   * property of the VirtualNetworkGateway resource.
   * @property {string} [virtualNetworkGateway1.provisioningState] The
   * provisioning state of the VirtualNetworkGateway resource. Possible values
   * are: 'Updating', 'Deleting', and 'Failed'.
   * @property {string} [virtualNetworkGateway1.etag] Gets a unique read-only
   * string that changes whenever the resource is updated.
   * @property {object} [virtualNetworkGateway2] The reference to virtual
   * network gateway resource.
   * @property {array} [virtualNetworkGateway2.ipConfigurations] IP
   * configurations for virtual network gateway.
   * @property {string} [virtualNetworkGateway2.gatewayType] The type of this
   * virtual network gateway. Possible values are: 'Vpn' and 'ExpressRoute'.
   * Possible values include: 'Vpn', 'ExpressRoute'
   * @property {string} [virtualNetworkGateway2.vpnType] The type of this
   * virtual network gateway. Possible values are: 'PolicyBased' and
   * 'RouteBased'. Possible values include: 'PolicyBased', 'RouteBased'
   * @property {boolean} [virtualNetworkGateway2.enableBgp] Whether BGP is
   * enabled for this virtual network gateway or not.
   * @property {boolean} [virtualNetworkGateway2.activeActive] ActiveActive
   * flag
   * @property {object} [virtualNetworkGateway2.gatewayDefaultSite] The
   * reference of the LocalNetworkGateway resource which represents local
   * network site having default routes. Assign Null value in case of removing
   * existing default site setting.
   * @property {string} [virtualNetworkGateway2.gatewayDefaultSite.id] Resource
   * ID.
   * @property {object} [virtualNetworkGateway2.sku] The reference of the
   * VirtualNetworkGatewaySku resource which represents the SKU selected for
   * Virtual network gateway.
   * @property {string} [virtualNetworkGateway2.sku.name] Gateway SKU name.
   * Possible values include: 'Basic', 'HighPerformance', 'Standard',
   * 'UltraPerformance', 'VpnGw1', 'VpnGw2', 'VpnGw3', 'VpnGw1AZ', 'VpnGw2AZ',
   * 'VpnGw3AZ', 'ErGw1AZ', 'ErGw2AZ', 'ErGw3AZ'
   * @property {string} [virtualNetworkGateway2.sku.tier] Gateway SKU tier.
   * Possible values include: 'Basic', 'HighPerformance', 'Standard',
   * 'UltraPerformance', 'VpnGw1', 'VpnGw2', 'VpnGw3', 'VpnGw1AZ', 'VpnGw2AZ',
   * 'VpnGw3AZ', 'ErGw1AZ', 'ErGw2AZ', 'ErGw3AZ'
   * @property {number} [virtualNetworkGateway2.sku.capacity] The capacity.
   * @property {object} [virtualNetworkGateway2.vpnClientConfiguration] The
   * reference of the VpnClientConfiguration resource which represents the P2S
   * VpnClient configurations.
   * @property {object}
   * [virtualNetworkGateway2.vpnClientConfiguration.vpnClientAddressPool] The
   * reference of the address space resource which represents Address space for
   * P2S VpnClient.
   * @property {array}
   * [virtualNetworkGateway2.vpnClientConfiguration.vpnClientAddressPool.addressPrefixes]
   * A list of address blocks reserved for this virtual network in CIDR
   * notation.
   * @property {array}
   * [virtualNetworkGateway2.vpnClientConfiguration.vpnClientRootCertificates]
   * VpnClientRootCertificate for virtual network gateway.
   * @property {array}
   * [virtualNetworkGateway2.vpnClientConfiguration.vpnClientRevokedCertificates]
   * VpnClientRevokedCertificate for Virtual network gateway.
   * @property {array}
   * [virtualNetworkGateway2.vpnClientConfiguration.vpnClientProtocols]
   * VpnClientProtocols for Virtual network gateway.
   * @property {array}
   * [virtualNetworkGateway2.vpnClientConfiguration.vpnClientIpsecPolicies]
   * VpnClientIpsecPolicies for virtual network gateway P2S client.
   * @property {string}
   * [virtualNetworkGateway2.vpnClientConfiguration.radiusServerAddress] The
   * radius server address property of the VirtualNetworkGateway resource for
   * vpn client connection.
   * @property {string}
   * [virtualNetworkGateway2.vpnClientConfiguration.radiusServerSecret] The
   * radius secret property of the VirtualNetworkGateway resource for vpn
   * client connection.
   * @property {object} [virtualNetworkGateway2.bgpSettings] Virtual network
   * gateway's BGP speaker settings.
   * @property {number} [virtualNetworkGateway2.bgpSettings.asn] The BGP
   * speaker's ASN.
   * @property {string} [virtualNetworkGateway2.bgpSettings.bgpPeeringAddress]
   * The BGP peering address and BGP identifier of this BGP speaker.
   * @property {number} [virtualNetworkGateway2.bgpSettings.peerWeight] The
   * weight added to routes learned from this BGP speaker.
   * @property {object} [virtualNetworkGateway2.customRoutes] The reference of
   * the address space resource which represents the custom routes address
   * space specified by the the customer for virtual network gateway and
   * VpnClient.
   * @property {array} [virtualNetworkGateway2.customRoutes.addressPrefixes] A
   * list of address blocks reserved for this virtual network in CIDR notation.
   * @property {string} [virtualNetworkGateway2.resourceGuid] The resource GUID
   * property of the VirtualNetworkGateway resource.
   * @property {string} [virtualNetworkGateway2.provisioningState] The
   * provisioning state of the VirtualNetworkGateway resource. Possible values
   * are: 'Updating', 'Deleting', and 'Failed'.
   * @property {string} [virtualNetworkGateway2.etag] Gets a unique read-only
   * string that changes whenever the resource is updated.
   * @property {object} [localNetworkGateway2] The reference to local network
   * gateway resource.
   * @property {object} [localNetworkGateway2.localNetworkAddressSpace] Local
   * network site address space.
   * @property {array}
   * [localNetworkGateway2.localNetworkAddressSpace.addressPrefixes] A list of
   * address blocks reserved for this virtual network in CIDR notation.
   * @property {string} [localNetworkGateway2.gatewayIpAddress] IP address of
   * local network gateway.
   * @property {object} [localNetworkGateway2.bgpSettings] Local network
   * gateway's BGP speaker settings.
   * @property {number} [localNetworkGateway2.bgpSettings.asn] The BGP
   * speaker's ASN.
   * @property {string} [localNetworkGateway2.bgpSettings.bgpPeeringAddress]
   * The BGP peering address and BGP identifier of this BGP speaker.
   * @property {number} [localNetworkGateway2.bgpSettings.peerWeight] The
   * weight added to routes learned from this BGP speaker.
   * @property {string} [localNetworkGateway2.resourceGuid] The resource GUID
   * property of the LocalNetworkGateway resource.
   * @property {string} [localNetworkGateway2.provisioningState] The
   * provisioning state of the LocalNetworkGateway resource. Possible values
   * are: 'Updating', 'Deleting', and 'Failed'.
   * @property {string} [localNetworkGateway2.etag] A unique read-only string
   * that changes whenever the resource is updated.
   * @property {string} connectionType Gateway connection type. Possible values
   * include: 'IPsec', 'Vnet2Vnet', 'ExpressRoute', 'VPNClient'
   * @property {string} [connectionProtocol] Connection protocol used for this
   * connection. Possible values include: 'IKEv2', 'IKEv1'
   * @property {number} [routingWeight] The routing weight.
   * @property {string} [sharedKey] The IPSec shared key.
   * @property {string} [connectionStatus] Virtual Network Gateway connection
   * status. Possible values include: 'Unknown', 'Connecting', 'Connected',
   * 'NotConnected'
   * @property {array} [tunnelConnectionStatus] Collection of all tunnels'
   * connection health status.
   * @property {number} [egressBytesTransferred] The egress bytes transferred
   * in this connection.
   * @property {number} [ingressBytesTransferred] The ingress bytes transferred
   * in this connection.
   * @property {object} [peer] The reference to peerings resource.
   * @property {string} [peer.id] Resource ID.
   * @property {boolean} [enableBgp] EnableBgp flag
   * @property {boolean} [usePolicyBasedTrafficSelectors] Enable policy-based
   * traffic selectors.
   * @property {array} [ipsecPolicies] The IPSec Policies to be considered by
   * this connection.
   * @property {string} [resourceGuid] The resource GUID property of the
   * VirtualNetworkGatewayConnection resource.
   * @property {string} [provisioningState] The provisioning state of the
   * VirtualNetworkGatewayConnection resource. Possible values are: 'Updating',
   * 'Deleting', and 'Failed'.
   * @property {boolean} [expressRouteGatewayBypass] Bypass ExpressRoute
   * Gateway for data forwarding
   * @property {string} [etag] Gets a unique read-only string that changes
   * whenever the resource is updated.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of VirtualNetworkGatewayConnection
   *
   * @returns {object} metadata of VirtualNetworkGatewayConnection
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'VirtualNetworkGatewayConnection',
      type: {
        name: 'Composite',
        className: 'VirtualNetworkGatewayConnection',
        modelProperties: {
          id: {
            required: false,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          authorizationKey: {
            required: false,
            serializedName: 'properties.authorizationKey',
            type: {
              name: 'String'
            }
          },
          virtualNetworkGateway1: {
            required: true,
            serializedName: 'properties.virtualNetworkGateway1',
            type: {
              name: 'Composite',
              className: 'VirtualNetworkGateway'
            }
          },
          virtualNetworkGateway2: {
            required: false,
            serializedName: 'properties.virtualNetworkGateway2',
            type: {
              name: 'Composite',
              className: 'VirtualNetworkGateway'
            }
          },
          localNetworkGateway2: {
            required: false,
            serializedName: 'properties.localNetworkGateway2',
            type: {
              name: 'Composite',
              className: 'LocalNetworkGateway'
            }
          },
          connectionType: {
            required: true,
            serializedName: 'properties.connectionType',
            type: {
              name: 'String'
            }
          },
          connectionProtocol: {
            required: false,
            serializedName: 'properties.connectionProtocol',
            type: {
              name: 'String'
            }
          },
          routingWeight: {
            required: false,
            serializedName: 'properties.routingWeight',
            type: {
              name: 'Number'
            }
          },
          sharedKey: {
            required: false,
            serializedName: 'properties.sharedKey',
            type: {
              name: 'String'
            }
          },
          connectionStatus: {
            required: false,
            readOnly: true,
            serializedName: 'properties.connectionStatus',
            type: {
              name: 'String'
            }
          },
          tunnelConnectionStatus: {
            required: false,
            readOnly: true,
            serializedName: 'properties.tunnelConnectionStatus',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'TunnelConnectionHealthElementType',
                  type: {
                    name: 'Composite',
                    className: 'TunnelConnectionHealth'
                  }
              }
            }
          },
          egressBytesTransferred: {
            required: false,
            readOnly: true,
            serializedName: 'properties.egressBytesTransferred',
            type: {
              name: 'Number'
            }
          },
          ingressBytesTransferred: {
            required: false,
            readOnly: true,
            serializedName: 'properties.ingressBytesTransferred',
            type: {
              name: 'Number'
            }
          },
          peer: {
            required: false,
            serializedName: 'properties.peer',
            type: {
              name: 'Composite',
              className: 'SubResource'
            }
          },
          enableBgp: {
            required: false,
            serializedName: 'properties.enableBgp',
            type: {
              name: 'Boolean'
            }
          },
          usePolicyBasedTrafficSelectors: {
            required: false,
            serializedName: 'properties.usePolicyBasedTrafficSelectors',
            type: {
              name: 'Boolean'
            }
          },
          ipsecPolicies: {
            required: false,
            serializedName: 'properties.ipsecPolicies',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'IpsecPolicyElementType',
                  type: {
                    name: 'Composite',
                    className: 'IpsecPolicy'
                  }
              }
            }
          },
          resourceGuid: {
            required: false,
            serializedName: 'properties.resourceGuid',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          expressRouteGatewayBypass: {
            required: false,
            serializedName: 'properties.expressRouteGatewayBypass',
            type: {
              name: 'Boolean'
            }
          },
          etag: {
            required: false,
            serializedName: 'etag',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = VirtualNetworkGatewayConnection;
